/*
  "MoonSun"
  calculates Moon's and Sun's position, time of rise and set + Moon's phase for a given UTC (time_t) and coordinate
  with "double"-precision, if available

  derived from Paul Schlyter's http://www.stjarnhimlen.se

  Requires Michael Margolis library "Time"
  
  V1.3 / Copyright (C) 2023, T.Rode (DL1DUZ)

  Permission is granted to use, copy, modify, and distribute this software
  and documentation for non-commercial purposes.
*/

#ifndef MoonSun_h_
#define MoonSun_h_

#if (ARDUINO >= 100)
#include <Arduino.h> 
#else
#include <WProgram.h> 
#endif

#include <stdint.h>
#include <math.h>
#include <TimeLib.h>

//################################################################################################################
//definitions
//################################################################################################################

// structure containing an array of 13 "double"-variables
struct dset {
  double x[13];
};

//################################################################################################################

class MoonSunClass {

public:

  dset Position(const time_t time, const uint8_t lat_deg, const uint8_t lat_min, const uint8_t lat_sec,
                const uint8_t lat_o, const uint8_t long_deg, const uint8_t long_min, const uint8_t long_sec,
				const uint8_t long_o);
  time_t RiseSet(const uint8_t selection, time_t time, const uint8_t lat_deg, const uint8_t lat_min,
                 const uint8_t lat_sec, const uint8_t lat_o, const uint8_t long_deg, const uint8_t long_min,
				 const uint8_t long_sec, const uint8_t long_o);
  int32_t radToDegree(const double angle);

private:

  // some constants
  const double M_PI_12 = M_PI/12;
  const double M_PI_180 = M_PI/180;
  const double _12_M_PI = 12/M_PI;

  double corrAngle(double angle);
  dset calDaTA(const double e, const double M, const double a);
  uint32_t getUTC_s(const time_t time);

};

extern MoonSunClass MS;

#endif // MoonSun_h_
